"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }// src/index.ts


var _fs = require('fs');
var _debug = require('debug'); var _debug2 = _interopRequireDefault(_debug);

// src/resolveDevice.ts
var cache = new Map();
function resolveDevice(req, devices) {
  const userAgent = req.headers["user-agent"];
  if (!userAgent)
    return;
  const cached = cache.get(userAgent);
  if (cached) {
    return cached;
  }
  const result = Object.keys(devices).find((device) => {
    const re = devices[device];
    if (typeof re === "boolean") {
      return re;
    }
    if (typeof re === "function") {
      return re(req);
    }
    return re.test(userAgent);
  });
  if (result)
    cache.set(userAgent, result);
  return result;
}

// src/utils.ts
var _path = require('path');
function idToFilePath(id, root) {
  if (id[0] !== "/" || id[1] === "@")
    return false;
  const filePath = id.startsWith(root) ? id : _path.resolve.call(void 0, root, id.slice(1).replace(/\?.+$/, ""));
  if (!filePath.match(/\.(.+)$/i)) {
    return false;
  }
  return filePath;
}
function resolveDeviceFilePath(filePath, device) {
  return filePath.replace(/\.([^?]+)(\?.*)?$/, `.${device}.$1$2`);
}

// src/index.ts
var defaultOptions = {
  devices: {
    mobile: /Mobile|iP(hone|od|ad)|Android|BlackBerry|IEMobile|Kindle|NetFront|Silk-Accelerated|(hpw|web)OS|Fennec|Minimo|Opera M(obi|ini)|Blazer|Dolfin|Dolphin|Skyfire|Zune/,
    desktop: true
  },
  buildFor: process.env.DEVICE || "desktop",
  id: "Object.DEVICE"
};
var debug = _debug2.default.call(void 0, "vite:multi-device");
function multiDevice(options) {
  options = {...defaultOptions, ...options};
  let {buildFor} = options;
  if (!buildFor) {
    buildFor = Object.keys(options.devices)[0];
  }
  const PACKAGE_NAME = "@xii/vite-plugin-multi-device";
  const DEVICE_PACKAGE_NAME = PACKAGE_NAME + "/device";
  let config;
  return [{
    name: "vite:multi-device",
    enforce: "pre",
    configResolved(_config) {
      config = _config;
    },
    configureServer({app}) {
      app.use((req, res, next) => {
        const {url} = req;
        if (url === "/@id/" + DEVICE_PACKAGE_NAME) {
          const device = resolveDevice(req, options.devices);
          res.setHeader("Content-Type", "application/javascript");
          res.end(`
                    ${options.id} = { ${device}: true }
                    `);
          debug(`apply global property ${options.id}`);
          return;
        }
        if (url && url[0] === "/" && url[1] !== "@") {
          const device = resolveDevice(req, options.devices);
          const filePath = idToFilePath(url, config.root);
          if (filePath && device) {
            const deviceFilePath = resolveDeviceFilePath(filePath, device);
            if (_fs.existsSync.call(void 0, deviceFilePath)) {
              const newUrl = resolveDeviceFilePath(url, device);
              res.setHeader("Content-Type", "application/javascript");
              res.end(`
                            import Component from '${newUrl}';
                            export default Component;
                            export * from '${newUrl}';
                            `.trim());
              debug(`dev server bridge "${url}" -> "${newUrl}"`);
              return;
            }
          }
        }
        return next();
      });
    },
    resolveId(id) {
      if (id === DEVICE_PACKAGE_NAME) {
        return DEVICE_PACKAGE_NAME;
      }
      const filePath = idToFilePath(id, config.root);
      if (filePath) {
        const deviceFilePath = resolveDeviceFilePath(filePath, buildFor);
        if (_fs.existsSync.call(void 0, deviceFilePath)) {
          debug(`switch id: "${id}" -> "${deviceFilePath}"`);
          return deviceFilePath;
        }
      }
    },
    load(id) {
      if (id === DEVICE_PACKAGE_NAME) {
        debug(`return empty: ${DEVICE_PACKAGE_NAME}`);
        return "";
      }
    }
  }, {
    name: "vite:multi-device-build",
    apply: "build",
    config() {
      return {
        define: Object.keys(options.devices).reduce((carry, item) => {
          carry[options.id + "." + item] = buildFor === item;
          return carry;
        }, {})
      };
    }
  }];
}


exports.default = multiDevice;
